(load-relative "../uri-match.scm")

(import uri-match)
(use test srfi-1 data-structures uri-common)

(test-group "routes creation"
  (let ([routes (make-routes '((get "foo")
			       ("/foo"
				(get "bar")
				(post "posted")
				("/bar" (get "nested")))
			       (post "ha!")))])

    (test-assert (lset= '(get post) (map car routes)))

    (test-group "expansion"

      (test-assert (lset= '("/" "/foo" "/foo/bar")
			  (map car (alist-ref 'get routes))))

      (test-assert (lset= '("/" "/foo")
			  (map car (alist-ref 'post routes)))))))


(test-group "basic matching"

  (test "this is the body"
	((uri-match 'get "/" (make-routes '((get "this is the body"))))))

  (test "against the path of a uri-reference" "something!"
	((uri-match 'get (uri-reference "http://foo/bar") (make-routes '(("/bar" (get "something!")))))))

  (test-assert (not (uri-match 'get "/" (make-routes '((post "won't reach me"))))))

  (test-group "with nesting"
    (test-assert (equal? "foo" ((uri-match 'post "/foo/bar" (make-routes '(("/foo" ("/bar" (post "foo")))))))))

    (test-group "with procedure body"
      (test-assert (eq? 'something
			((uri-match 'get "/me" (make-routes `(("/me" (get ,(lambda () 'something)))))))))

      (test-assert (= 100 ((uri-match 'get "/numbers/100"
				      (make-routes `(("/numbers/(\\d+)" (get ,(lambda (n) (string->number n))))))))))))

  (test-group "with capture groups"
    (let ([routes (make-routes `(("/foo" ("/(\\d+)" ("/(\\d+)" (get ,string-append))))))])
      
      (test-assert (string= "105" ((uri-match 'get "/foo/10/5" routes))))
      (test-assert (not (uri-match 'get "/foo/bar/10" routes))))))


(test-group "matcher" 
	  
  (let ([matcher (make-uri-matcher '((get "is") ("/this" (post "it") ("/or" (put "what?")))))])

    (test-assert (equal? "is" ((matcher 'get "/"))))

    (test-assert (equal? "it" ((matcher 'post "/this"))))

    (test-assert (equal? "what?" ((matcher 'put "/this/or"))))))